import { defineComponent, reactive, computed, PropType } from 'vue';
import { useCommonStore } from '@/store';
import { UploadData } from '@/api/common';
import './index.scss';
import Preview from '../Preview';
import { isImage, isVideo, sleep } from '@/utils';
import { update } from 'lodash';
import { BaseMediaUrl } from '@/constants/constants';
import { Notify, Uploader } from 'vant';
interface State {
  dialog: boolean;
  previewCurrentKey: number;
  localFiles: UploadData[];
  uploadFiles: UploadData[];
  loading: boolean;
}

export default defineComponent({
  props: {
    media: String,
    mediaType: {
      type: String as PropType<'image' | 'video'>,
      default: 'image',
    },
    accept: String,
  },

  setup(props, ctx) {
    const commonStore = useCommonStore();

    const state = reactive<State>({
      dialog: false,
      previewCurrentKey: 0,
      localFiles: [],
      uploadFiles:
        props.media?.split(',').map((i) => ({ url: i, fileName: i })) ?? [],
      loading: false,
    });

    const allFiles = computed(() =>
      ([] as UploadData[])
        .concat(
          ...state.uploadFiles
            ?.filter((media) =>
              props.mediaType === 'image' ? isImage(media) : isVideo(media),
            )
            .concat(...state.localFiles),
        )
        .map((i) => ({ ...i, type: props.mediaType })),
    );

    const handlePreview = (idx: number) => {
      state.previewCurrentKey = idx;
      state.dialog = true;
    };

    const handleUpload = async (file: File) => {
      if (file.name.length > 100) {
        return Notify({ type: 'danger', message: '上传的文件名最长100个字符' });
      }
      state.localFiles.push({
        url: window.URL.createObjectURL(file),
        fileName: file.name,
      });
      state.loading = true;
      console.log(window.URL.createObjectURL(file), '+++');

      const data = await commonStore.upload(file);
      await sleep(1000);
      state.uploadFiles.push(data);
      state.loading = false;
      state.localFiles = [];
      console.log(allFiles.value);

      ctx.emit('update:media', allFiles.value.map((i) => i.url).join());
    };

    return () => (
      <div class="upload-file-list-container">
        {allFiles.value.map((file, idx) => {
          return (
            <div
              key={`${file.fileName}_${idx}`}
              class="file-item"
              v-loading={file.url?.includes('blob:') && state.loading}>
              {file.url && props.mediaType === 'image' ? (
                <img
                  src={
                    file.url?.includes('blob:')
                      ? file.url
                      : BaseMediaUrl + file.url
                  }
                  alt={file.fileName}
                />
              ) : (
                <video>
                  <source
                    src={
                      file.url?.includes('blob:')
                        ? file.url
                        : BaseMediaUrl + file.url
                    }
                  />
                </video>
              )}
              <div class="file-action">
                <i class="el-icon-view" onClick={() => handlePreview(idx)} />
              </div>
              <div class="file-delete-btn">
                {/* <el-popconfirm
                  title="确定删除这个文件吗?"
                  onConfirm={() => {}}
                  v-slots={{
                    reference: () => (
                      <i class="el-icon-close" onClick={() => {}} />
                    ),
                  }}
                /> */}
                      <Icon name="cross" />

              </div>
            </div>
          );
        })}
        <Uploader
          multiple
          action={''}
          accept={props.accept}
          beforeRead={(file: File) => {
            handleUpload(file);
            return false;
          }}>
          <div class="upload-btn">
            {ctx.slots.default && ctx.slots.default()}
          </div>
        </el-upload>
        <Preview
          current={state.previewCurrentKey}
          medias={allFiles.value}
          v-model:dialog={state.dialog}
        />
      </div>
    );
  },
});
